# Takes an image file directory and makes the web page that lists them.
# They are listed in creation date/time order.

import glob
import os

def getFiles() :
    files = glob.glob("C:\\OpenSourceGit\\AudacityTeamTools\\wit-html\\auto_images\\*.png")
    files.sort(key=os.path.getmtime)
    return [ os.path.basename( name ) for name in files ]
    #print("\n".join(files))

def oneItem( name ) :
    return "<img src='./auto_images/"+name+"'><br><em>"+name+"</em><br>"

files = getFiles()
print( "\n".join( [ oneItem(name) for name in files ] ) )



from debug_output import *

# Make a plot of the ODF, with peaks marked. Add another layer where the tatums are shown, for example as vertical lines.
import matplotlib.pyplot as plt
import math

plt.figure(1)
# Plot the ODF, using odfSr to convert from samples to seconds
plt.plot([i/odfSr for i in range(len(odf))], odf)
# Plot the peaks
plt.plot([i/odfSr for i in odf_peak_indices], [odf[i] for i in odf_peak_indices], 'ro')
tatumsPerSecond = tatumRate / 60.0

# Plot the tatums as vertical lines, offset by `lag` samples
numTatums = int(math.ceil(len(odf) / odfSr * tatumsPerSecond))
for i in range(numTatums):
    plt.axvline(x=i/tatumsPerSecond + lag / odfSr, color='g', linestyle='--')

plt.xlim(0, len(odf) / odfSr)
plt.title(wavFile)

# Plot `rawOdf` and `movingAverage` on the same plot.
plt.figure(2)
plt.plot([i/odfSr for i in range(len(odf))], rawOdf)
plt.plot([i/odfSr for i in range(len(odf))], movingAverage)
plt.xlim(0, len(odf) / odfSr)
plt.grid(True)
plt.title(wavFile)

# Plot `odfAutoCorr`, whose length is half that of `odf` + 1. Add a layer where the peaks at `odfAutoCorrPeakIndices` are marked.
plt.figure(3)
plt.plot([i/odfSr for i in range(len(odfAutoCorr))], odfAutoCorr)
plt.plot([i/odfSr for i in odfAutoCorrPeakIndices], [odfAutoCorr[i] for i in odfAutoCorrPeakIndices], 'ro')
plt.xlim(0, len(odfAutoCorr) / odfSr)
plt.grid(True)
plt.title(wavFile)

plt.show()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests the audacity pipe.

Keep pipe_test.py short!!
You can make more complicated longer tests to test other functionality
or to generate screenshots etc in other scripts.

Make sure Audacity is running first and that mod-script-pipe is enabled
before running this script.

Requires Python 2.7 or later. Python 3 is strongly recommended.

"""

import os
import sys


if sys.platform == 'win32':
    print("pipe-test.py, running on windows")
    TONAME = '\\\\.\\pipe\\ToSrvPipe'
    FROMNAME = '\\\\.\\pipe\\FromSrvPipe'
    EOL = '\r\n\0'
else:
    print("pipe-test.py, running on linux or mac")
    TONAME = '/tmp/audacity_script_pipe.to.' + str(os.getuid())
    FROMNAME = '/tmp/audacity_script_pipe.from.' + str(os.getuid())
    EOL = '\n'

print("Write to  \"" + TONAME +"\"")
if not os.path.exists(TONAME):
    print(" ..does not exist.  Ensure Audacity is running with mod-script-pipe.")
    sys.exit()

print("Read from \"" + FROMNAME +"\"")
if not os.path.exists(FROMNAME):
    print(" ..does not exist.  Ensure Audacity is running with mod-script-pipe.")
    sys.exit()

print("-- Both pipes exist.  Good.")

TOFILE = open(TONAME, 'w')
print("-- File to write to has been opened")
FROMFILE = open(FROMNAME, 'rt')
print("-- File to read from has now been opened too\r\n")


def send_command(command):
    """Send a single command."""
    print("Send: >>> \n"+command)
    TOFILE.write(command + EOL)
    TOFILE.flush()

def get_response():
    """Return the command response."""
    result = ''
    line = ''
    while True:
        result += line
        line = FROMFILE.readline()
        if line == '\n' and len(result) > 0:
            break
    return result

def do_command(command):
    """Send one command, and return the response."""
    send_command(command)
    response = get_response()
    print("Rcvd: <<< \n" + response)
    return response

def quick_test():
    """Example list of commands."""
    do_command('Help: Command=Help')
    do_command('Help: Command="GetInfo"')
    #do_command('SetPreference: Name=GUI/Theme Value=classic Reload=1')

quick_test()



import subprocess
import json

from helpers.conan_environment import get_conan

def list_remotes():
    remotes = subprocess.check_output([get_conan(), 'remote', 'list', '--format', 'json'], stdin=subprocess.DEVNULL).decode('utf-8')
    return json.loads(remotes)


def remove_remote(remote_name:str):
    remotes = list_remotes()
    for remote in remotes:
        if remote['name'] != remote_name:
            continue
        return subprocess.check_call([get_conan(), 'remote', 'remove', remote_name], stdin=subprocess.DEVNULL)


def add_remote(name:str, url:str) -> None:
    remotes = list_remotes()
    for remote in remotes:
        if remote['name'] != name:
            continue
        if remote['url'] != url:
            subprocess.check_call([get_conan(), 'remote', 'update', '--url', name], stdin=subprocess.DEVNULL)
        return

    subprocess.check_call([get_conan(), 'remote', 'add', name, url], stdin=subprocess.DEVNULL)


def validate_remotes():
    old_remotes = 'audacity', 'conan-center-cache', 'audacity-recipes', 'audacity-binaries'

    for remote in old_remotes:
        try:
            remove_remote(remote)
        finally:
            pass

    add_remote('audacity-recipes-conan2', 'https://artifactory.audacityteam.org/artifactory/api/conan/audacity-recipes-conan2')
    add_remote('audacity-binaries-conan2', 'https://artifactory.audacityteam.org/artifactory/api/conan/audacity-binaries-conan2')



import os
import re
import argparse

parser = argparse.ArgumentParser()

parser.add_argument("--input")
parser.add_argument("--include-dir", nargs="+")
parser.add_argument("--output")

args = parser.parse_args()

system_includes = set()
definitions = set()


def resolve_path(file_name, relative_to):
    if os.path.isabs(file_name):
        return file_name

    base_dir = os.path.dirname(relative_to)
    result = os.path.join(base_dir, file_name)

    if os.path.isfile(result):
        return result

    for include_dir in args.include_dir:
        result = os.path.join(include_dir, file_name)
        if os.path.isfile(result):
            return os.path.abspath(result)

    return None


def cleanup_comments(lines):
    single_line_comment = re.compile(r'(?://.*)|(?:/\*.*\*/)')

    multiline_comment_start = re.compile(r'/\*.*')
    multiline_comment_end = re.compile(r'.*\*/')

    output = []

    in_multiline_comment = False

    for line in lines:
        if in_multiline_comment:
            if multiline_comment_end.match(line):
                in_multiline_comment = False
                output.append(re.sub(multiline_comment_end, '', line))
        else:
            line = re.sub(single_line_comment, '', line)

            if multiline_comment_start.search(line):
                in_multiline_comment = True
                line = re.sub(multiline_comment_start, '', line)

            output.append(line)

    return output


def is_definition_enabled(definition):
    return definition in definitions


def get_definition(match_result):
    for group in match_result.groups():
        if group:
            return group

    return None


def process_include(line, base_path):
    output = []

    include_re = re.compile(r'^#\s*include\s+"(.+)"\s*$')

    match = include_re.match(line)

    if not match:
        output.append(line)
    else:
        include_name = match.group(1)
        full_file_path = resolve_path(include_name, base_path)

        if full_file_path is not None:
            output = preprocess_file(resolve_path(include_name, base_path))

    return output


def cleanup_ifs(lines, base_path):
    output = []

    stack = [{
        "context_enabled": True
    }]

    def is_context_enabled():
        for ctx in stack:
            if not ctx["context_enabled"]:
                return False

        return True

    positive_re = re.compile(r'^(?:\s*(?:#\s*ifdef\s+(.+))|(?:#\s*if\s+([^!].*))|(?:#\s*elif\s+([^!].*)))$')
    negative_re = re.compile(r'^(?:\s*(?:#\s*ifndef\s+(.+))|(?:#\s*if\s+!(.+))|(?:#\s*elif\s+!(.+)))$')
    else_re = re.compile(r'^\s*#\s*else\s*$')
    endif_re = re.compile(r'^\s*#\s*endif\s*$')
    define_re = re.compile(r'^\s*#\s*define\s+([A-Za-z0-9_]+)(?:\s.*)?$')

    for line in lines:
        if positive_re.match(line):
            match = positive_re.match(line)
            definition = get_definition(match)

            if line.find("elif") < 0:
                stack.append({})

            stack[-1]["context_enabled"] = is_definition_enabled(definition)
        elif negative_re.match(line):
            match = negative_re.match(line)
            definition = get_definition(match)

            if line.find("elif") < 0:
                stack.append({})

            stack[-1]["context_enabled"] = not is_definition_enabled(definition)
        elif else_re.match(line):
            stack[-1]["context_enabled"] = not stack[-1]["context_enabled"]
        elif endif_re.match(line):
            stack.pop()
        elif is_context_enabled():
            define_match = define_re.match(line)

            if define_match is not None:
                definitions.add(define_match.group(1))

            output = output + process_include(line, base_path)

    return output


def gather_system_includes(lines):
    output = []

    include_re = re.compile(r'^#\s*include <([a-zA-Z0-9_.\/]*)>\s*$')

    for line in lines:
        match = include_re.match(line)

        if match:
            system_includes.add(match.group(1))
        else:
            output.append(line)

    return output



def preprocess_file(filename):
    output_lines = []

    with open(filename, 'r') as f:
        output_lines = f.readlines()

    output_lines = cleanup_comments(output_lines)
    output_lines = cleanup_ifs(output_lines, filename)

    return gather_system_includes(output_lines)


with open(args.output, "w") as outfile:
    outfile.write("// This header was generated from the FFMPEG headers\n")
    outfile.write("#pragma once\n\n")

    output = preprocess_file(os.path.abspath(args.input))

    for global_include in system_includes:
        outfile.write("#include <{}>\n".format(global_include))

    outfile.write("\n")

    prev_len = 0

    for line in output:
        curr_len = len(line.strip())

        if curr_len > 0 or prev_len > 0:
            outfile.write(line)

        prev_len = curr_len



#! /usr/bin/python3
# -*- coding: utf-8 -*-

"""Test Import / Export and recording.

recording-test.py loads a WAV file, plays it, recording at the same time until
the end of the track, and then exports the recording as a WAV with "-out"
appended to the file name.

To run the test without input prompts, set valid values for
PATH and INFILE.

User supplied variables
-------
    PATH: Path to the folder containing the input test file. Also used for exporting the result.
    INFILE: Name of the input WAV file.

With a little modification, can be suitable for rinse and repeat with different
input files.

Make sure Audacity is running and that mod-script-pipe is enabled
before running this script.
"""

import os
import sys
import time
import json


# Platform specific file name and file path.
# PATH is the location of files to be imported / exported.

#PATH = './'
PATH = ""
while not os.path.isdir(PATH):
    PATH = os.path.realpath(input('Path to test folder: '))
    if not os.path.isdir(PATH):
        print('Invalid path. Try again.')
print('Test folder: ' + PATH)


#INFILE = "testfile.wav"
INFILE = ""
while not os.path.isfile(os.path.join(PATH, INFILE)):
    INFILE = input('Name of input WAV file: ')
    # Ensure we have the .wav extension.
    INFILE = os.path.splitext(INFILE)[0] + '.wav'
    if not os.path.isfile(os.path.join(PATH, INFILE)):
        print(f"{os.path.join(PATH, INFILE)} not found. Try again.")
    else:
        print(f"Input file: {os.path.join(PATH, INFILE)}")
# Remove file extension.
INFILE = os.path.splitext(INFILE)[0]


# Platform specific constants
if sys.platform == 'win32':
    print("recording-test.py, running on windows")
    PIPE_TO_AUDACITY = '\\\\.\\pipe\\ToSrvPipe'
    PIPE_FROM_AUDACITY = '\\\\.\\pipe\\FromSrvPipe'
    EOL = '\r\n\0'
else:
    print("recording-test.py, running on linux or mac")
    PIPE_TO_AUDACITY = '/tmp/audacity_script_pipe.to.' + str(os.getuid())
    PIPE_FROM_AUDACITY = '/tmp/audacity_script_pipe.from.' + str(os.getuid())
    EOL = '\n'


print("Write to  \"" + PIPE_TO_AUDACITY +"\"")
if not os.path.exists(PIPE_TO_AUDACITY):
    print(""" ..does not exist.
    Ensure Audacity is running with mod-script-pipe.""")
    sys.exit()

print("Read from \"" + PIPE_FROM_AUDACITY +"\"")
if not os.path.exists(PIPE_FROM_AUDACITY):
    print(""" ..does not exist.
    Ensure Audacity is running with mod-script-pipe.""")
    sys.exit()

print("-- Both pipes exist.  Good.")

TOPIPE = open(PIPE_TO_AUDACITY, 'w')
print("-- File to write to has been opened")
FROMPIPE = open(PIPE_FROM_AUDACITY, 'r')
print("-- File to read from has now been opened too\r\n")


def send_command(command):
    """Send a command to Audacity."""
    print("Send: >>> "+command)
    TOPIPE.write(command + EOL)
    TOPIPE.flush()


def get_response():
    """Get response from Audacity."""
    line = FROMPIPE.readline()
    result = ""
    while True:
        result += line
        line = FROMPIPE.readline()
        # print(f"Line read: [{line}]")
        if line == '\n':
            return result


def do_command(command):
    """Do the command. Return the response."""
    send_command(command)
    # time.sleep(0.1) # may be required on slow machines
    response = get_response()
    print("Rcvd: <<< " + response)
    return response


def play_record(filename):
    """Import track and record to new track.
    Note that a stop command is not required as playback will stop at end of selection.
    """
    do_command(f"Import2: Filename={os.path.join(PATH, filename + '.wav')}")
    do_command("Select: Track=0")
    do_command("SelTrackStartToEnd")
    # Our imported file has one clip. Find the length of it.
    clipsinfo = do_command("GetInfo: Type=Clips")
    clipsinfo = clipsinfo[:clipsinfo.rfind('BatchCommand finished: OK')]
    clips = json.loads(clipsinfo)
    duration = clips[0]['end'] - clips[0]['start']
    # Now we can start recording.
    do_command("Record2ndChoice")
    print('Sleeping until recording is complete...')
    time.sleep(duration + 0.1)


def export(filename):
    """Export the new track, and deleted both tracks."""
    do_command("Select: Track=1 mode=Set")
    do_command("SelTrackStartToEnd")
    do_command(f"Export2: Filename={os.path.join(PATH, filename)} NumChannels=1.0")
    do_command("SelectAll")
    do_command("RemoveTracks")


def do_one_file(name):
    """Run test with one input file only."""
    play_record(name)
    export(name + "-out.wav")


def quick_test():
    """Quick test to ensure pipe is working."""
    do_command('Help: CommandName=Help')


quick_test()
# Run the test with "testfile.wav" in the specified PATH.
do_one_file(INFILE)



# A functionality for developers.
# Disclaimer: At the time of writing only used on Windows; will not work out of the box on other OS.
#
# To use this:
# 1. Add
#       [TimeAndPitch]
#       TuningOn=1
#    to audacity.cfg (see https://manual.audacityteam.org/man/preferences.html#stored to find out where it is)
# 2. From the audacity.cfg directory, create `./TimeAndPitchTuning/overrideLogTime.txt`
# 3. Enter a time in seconds, e.g. (0.5)
# 4. Run your Audacity build and play a track with pitch shifting and formant preservation enabled.
#
# You will hear a gentle noise burst at the specified time, which indicates precisely the place where the snapshot was taken.
# Now you can run this script from lib-time-and-pitch's root directory.
#
# Note: the value in `overrideLogTime.txt` gets re-read on each playback, but to enable/disable the tuning,
# you have to change the config file and then restart Audacity.

import matplotlib.pyplot as plt
import os, sys

# Add the path to the local AppData\Roaming directory
appdata_path = os.path.join(os.getenv('APPDATA'), 'audacity', 'TimeAndPitchTuning')
sys.path.append(appdata_path)

import FormantShifterLog as FSL
import numpy as np

sampleRate = FSL.sampleRate
fftSize = FSL.fftSize
numBins = fftSize // 2 + 1
# Calculate `freqs` array using linspace
f = np.linspace(0, sampleRate / 2, numBins)
q = np.linspace(0, 1000*fftSize/sampleRate, fftSize)

yscale = 'log'
# yscale = 'linear'
markersize = 0
xlim = sampleRate/2

plt.figure()
plt.yscale(yscale)
plt.plot(f, FSL.weights)
plt.xlim(0, sampleRate/2)
plt.title('weights')
plt.grid()

plt.figure()
plt.title('fftSize:' + str(fftSize))

plt.subplot(2, 1, 1)
plt.yscale(yscale)
plt.plot(f, FSL.magnitude, marker='x', markersize=markersize)
plt.plot(f, FSL.envelope, linestyle='dotted', marker='o', markersize=markersize, fillstyle='none')
plt.legend(['mag', 'env'])
plt.xlim(0, xlim)
plt.ylabel('input')
plt.grid()

plt.subplot(2, 1, 2)
plt.yscale(yscale)
plt.plot(f, FSL.weightedMagnitude, marker='x', markersize=markersize)
plt.plot(f, FSL.envelopeResampled, linestyle='dotted', marker='o', markersize=markersize, fillstyle='none')
plt.legend(['mag', 'env'])
plt.xlim(0, xlim)
plt.ylabel('output')
plt.grid()

plt.figure()
plt.plot(q, FSL.cepstrum)
plt.plot(q, FSL.cepstrumLiftered, linestyle='dotted')
plt.title('cepstrum')
plt.grid()
plt.xlim(0, 1000*fftSize/sampleRate/2)
plt.xlabel('quefrency (ms)')

plt.show()



import os
import sys
import subprocess
import venv

from helpers.directories import directories
from helpers.version import version_tuple


def __get_venv_tool_path(tool_name:str):
    if sys.platform == 'win32':
        return os.path.join(os.environ['VIRTUAL_ENV'], 'Scripts', f'{tool_name}.exe')
    else:
        return os.path.join(os.environ['VIRTUAL_ENV'], 'bin', tool_name)


def get_python():
    if os.environ['VIRTUAL_ENV']:
        return __get_venv_tool_path('python')
    else:
        return sys.executable


class ConanEnv:
    old_env_path = None
    old_home_path = None

    def __init__(self, env_path:str=None, home_path:str=None):
        self.env_path = env_path

        if not self.env_path:
            expected_path = directories.env_dir
            if os.path.exists(expected_path):
                self.env_path = expected_path

        self.home_path = home_path

        if not self.home_path and directories.conan_home_dir:
            self.home_path = directories.conan_home_dir

    def __enter__(self):
        if self.env_path and 'VIRTUAL_ENV' in os.environ:
            self.old_env_path = os.environ['VIRTUAL_ENV']

        if self.home_path and 'CONAN_HOME' in os.environ:
            self.old_home_path = os.environ['CONAN_HOME']

        if self.env_path:
            os.environ['VIRTUAL_ENV'] = self.env_path

        if self.home_path:
            os.environ['CONAN_HOME'] = self.home_path

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.env_path:
            if self.old_env_path:
                os.environ['VIRTUAL_ENV'] = self.old_env_path
            else:
                del os.environ['VIRTUAL_ENV']

        if self.home_path:
            if self.old_home_path:
                os.environ['CONAN_HOME'] = self.old_home_path
            else:
                del os.environ['CONAN_HOME']


class ConanVenvBuilder(venv.EnvBuilder):
    def post_setup(self, context):
        with ConanEnv(context.env_dir):
            cmd = [get_python(), '-m', 'pip', 'install', 'conan']
            subprocess.check_call(cmd, stdin=subprocess.DEVNULL)


def get_conan():
    if 'VIRTUAL_ENV' in os.environ:
        return __get_venv_tool_path('conan')
    elif os.path.isdir(directories.env_dir):
        with ConanEnv():
            return __get_venv_tool_path('conan')
    else:
        if sys.platform == 'win32':
            return 'conan.exe'
        else:
            return 'conan'


def get_conan_version():
    try:
        version_string = subprocess.check_output([get_conan(), '--version'], stdin=subprocess.DEVNULL).decode('utf-8').strip()
        return version_tuple(version_string.split(' ')[-1])
    except FileNotFoundError:
        return None


def upgrade_conan():
    subprocess.check_call([get_python(), '-m', 'pip', 'install', '--upgrade', 'conan'], stdin=subprocess.DEVNULL)
    return get_conan_version()


def create_conan_venv(conan_venv:str=None):
    if not conan_venv:
        conan_venv = directories.env_dir

    print(f'Creating Conan virtual environment {conan_venv}')
    builder = ConanVenvBuilder(with_pip=True)
    builder.create(conan_venv)

    directories.conan_home_dir = os.path.join(directories.build_dir, 'conan', 'home')
    os.makedirs(directories.conan_home_dir, exist_ok=True)

    with ConanEnv(conan_venv):
        return get_conan_version()


def validate_conan(required_conan_version:tuple):
    conan_venv = directories.env_dir

    if os.path.isdir(conan_venv):
        print(f'Using Conan from local environment {conan_venv}')
        with ConanEnv(conan_venv):
            conan_version = get_conan_version()

            if conan_version is None:
                conan_version = create_conan_venv(conan_venv)
            elif conan_version < required_conan_version:
                conan_version = upgrade_conan()

            # Something went wrong during `pip install`?
            if conan_version is None or conan_version < required_conan_version:
                raise RuntimeError(f'Conan version {required_conan_version} is required')

            if not directories.conan_home_dir:
                default_home_dir = os.path.join(directories.build_dir, 'conan', 'home')
                if os.path.isdir(default_home_dir):
                    directories.conan_home_dir = default_home_dir


            return conan_version
    else:
        conan_version = get_conan_version()

        if conan_version is None:
            create_conan_venv(conan_venv)
            # Perfrorm full set of checks again
            return validate_conan(required_conan_version)
        elif conan_version < required_conan_version:
            raise RuntimeError(f'Conan version {conan_version} is too old, please upgrade to {required_conan_version} or newer')

        return conan_version



# get-gui-structure.py
# Obtains all menus and all box locations from the currently running Audacity.
# This is useful for wit.audacityteam.org, where we need these lists when we
# draw the menus and create the image maps.

# Make sure Audacity is running first and that mod-script-pipe is enabled
# before running this script.

import os
import sys


if( sys.platform  == 'win32' ):
    print( "get-gui-structure.py, running on windows" )
    toname = '\\\\.\\pipe\\ToSrvPipe'
    fromname = '\\\\.\\pipe\\FromSrvPipe'
    EOL = '\r\n\0'
else:
    print( "get-gui-structure.py, running on linux or mac" )
    toname = '/tmp/audacity_script_pipe.to.' + str(os.getuid())
    fromname = '/tmp/audacity_script_pipe.from.' + str(os.getuid())
    EOL = '\n'

print( "Write to  \"" + toname +"\"" )
if not os.path.exists( toname ) :
   print( " ..does not exist.  Ensure Audacity is running with mod-script-pipe." )
   sys.exit();
    
print( "Read from \"" + fromname +"\"")
if not os.path.exists( fromname ) :
   print( " ..does not exist.  Ensure Audacity is running with mod-script-pipe." )
   sys.exit();

print( "-- Both pipes exist.  Good." )

tofile = open( toname, 'wt+' )
print( "-- File to write to has been opened" )
fromfile = open( fromname, 'rt')
print( "-- File to read from has now been opened too\r\n" )

def sendCommand( command ) :
    print( "Send: >>> "+command )
    tofile.write( command + EOL )	
    tofile.flush()

def getResponse() :
    result = ''
    line = ''
    while line != '\n' :
        result += line
        line = fromfile.readline()
    return result

def doCommand( command ) :
    sendCommand( command )
    response = getResponse()
    print( "Rcvd: <<< " + response )
    return response

def do( command ) :
    doCommand( command )


def getStructure() :
    #do( 'Help: CommandName=Help' )
    #do( 'Help: CommandName=SetPreference' )
    #do( 'SetPreference: PrefName=GUI/Theme PrefValue=light' )
    do( 'GetInfo: Type=Boxes' )
    do( 'GetInfo: Type=Menus' )

getStructure()



from dataclasses import dataclass
from conan import ConanFile
from conan.tools.cmake import cmake_layout
from conan.tools.files import copy
import os
import subprocess

required_conan_version = ">=2.0.0"

# Fix expat rpah on macOS
def fix_expact_rpath(conanfile, filepath):
    if conanfile.settings.os != "Macos":
        return

    # Force expat ID to be @rpath/libexpat.dylib
    if 'expat' in filepath:
        filename = os.path.basename(filepath)
        conanfile.output.info(f"Setting id of {filepath} to @rpath/{filename}")
        subprocess.check_call(["install_name_tool", "-id", f"@rpath/{filename}", filepath])
        return

    deps = [dep.strip().split()[0] for dep in subprocess.check_output(["otool", "-L", filepath]).decode("utf-8").splitlines()]
    for dep in deps:
        if 'expat' not in dep:
            continue
        if not dep.startswith("@"):
            conanfile.output.info(f"=== Changing {dep} to @rpath/{os.path.basename(dep)} in {filepath} ===")
            subprocess.check_call(["install_name_tool", "-change", dep, f"@rpath/{os.path.basename(dep)}", filepath])

# A helper function that correctly copies the files from the Conan package to the
# correct location in the build tree
def global_copy_files(conanfile, dependency_info):
    if conanfile.settings.os == "Windows":
        if len(dependency_info.cpp_info.bindirs) == 0:
            return
        copy(conanfile, "*.dll", dependency_info.cpp_info.bindirs[0], f"{conanfile.build_folder}/{conanfile.settings.build_type}")
    elif conanfile.settings.os == "Macos":
        if len(dependency_info.cpp_info.libdirs) == 0:
            return

        copied_files = copy(conanfile, "*.dylib*", dependency_info.cpp_info.libdirs[0], f"{conanfile.build_folder}/Audacity.app/Contents/Frameworks")

        for file in copied_files:
            if not os.path.islink(file):
                try:
                    fix_expact_rpath(conanfile, file)
                except subprocess.CalledProcessError as e:
                    conanfile.output.error(f"Failed to set id of {file}: {e}")
    else:
        if len(dependency_info.cpp_info.libdirs) == 0:
            return
        # On Linux we also set the correct rpath for the copied libraries
        patchelf_path = os.path.join(conanfile.dependencies.build["patchelf"].cpp_info.bindirs[0], "patchelf")

        lib_dir = conanfile.options.lib_dir if conanfile.options.lib_dir else "lib/audacity"

        conanfile.output.info(f"Copying files from {dependency_info.cpp_info.libdirs[0]} to {conanfile.build_folder}/{lib_dir}")

        copied_files = copy(conanfile, "*.so*", dependency_info.cpp_info.libdirs[0], f"{conanfile.build_folder}/{lib_dir}")
        for file in copied_files:
            if not os.path.islink(file):
                subprocess.check_call([patchelf_path, "--add-rpath", "$ORIGIN", file], stdin=subprocess.DEVNULL)

# Dataclass that holds the information about a dependency
@dataclass
class AudacityDependency:
    name: str
    version: str
    channel: str = None
    package_options: dict = None
    default_enabled: bool = False
    override: bool = False

    def apply_options(self, conanfile, package):
        if self.package_options is not None:
            for key, value in self.package_options.items():
                conanfile.output.info(f"\t{self.name}:{key}={value}")
                setattr(package, key, value)

    def reference(self, conanfile):
        return f"{self.name}/{self.version}@{self.channel}" if self.channel else f"{self.name}/{self.version}@audacity/stable"

    def copy_files(self, conanfile, dependency_info):
        global_copy_files(conanfile, dependency_info)

# Dataclass that holds the information about the wxWidgets dependency
@dataclass
class wxWidgetsAudacityDependency:
    name: str = "wxwidgets"
    default_enabled: bool = False
    override: bool = False

    def reference(self, conanfile):
        return f"{self.name}/3.1.3.7-audacity@audacity/stable"

    def apply_options(self, conanfile, package):
        opts = [
            ("zlib", "zlib" if conanfile.options.use_zlib else "sys"),
            ("expat", "expat" if conanfile.options.use_expat else "sys"),
            ("png", "libpng" if conanfile.options.use_libpng else "sys"),
            ("jpeg", "libjpeg-turbo" if getattr(conanfile.options, "use_libjpeg-turbo") else "sys"),
            ("tiff", "off"),
            ("compatibility", 3.0),
            ("secretstore", False), ("opengl", False), ("propgrid", False), ("ribbon", False),
            ("richtext", False), ("stc", False), ("webview", False), ("help", False),
            ("html_help", False), ("fs_inet", False), ("protocol", False),
        ]

        for key, value in opts:
            conanfile.output.info(f"\t{self.name}:{key}={value}")
            setattr(package, key, value)

    def copy_files(self, conanfile, dependency_info):
        if conanfile.settings.os == "Windows":
            copy(conanfile, "*.dll", dependency_info.cpp_info.libdirs[0], f"{conanfile.build_folder}/{conanfile.settings.build_type}", keep_path=False)
        else:
            global_copy_files(conanfile, dependency_info)

@dataclass
class CrashpadDependency(AudacityDependency):
    def __init__(self, version: str, package_options: dict = None):
        super().__init__(name="crashpad", version=version, package_options=package_options)

    def copy_files(self, conanfile, dependency_info):
        crashpad_handler_filename = "crashpad_handler"
        if conanfile.settings.os == "Windows":
            crashpad_handler_filename += ".exe"

        dst_path = f"{conanfile.build_folder}"
        if conanfile.settings.os == "Windows":
            dst_path += f"/{conanfile.settings.build_type}"
        elif conanfile.settings.os == "Macos":
            dst_path += "/Audacity.app/Contents/MacOS"
        else:
            dst_path += "/bin"

        copy(conanfile, crashpad_handler_filename, dependency_info.cpp_info.bindirs[0], dst_path, keep_path=False)
        super().copy_files(conanfile, dependency_info)

# PortAudio has addittional options that need to be set
@dataclass
class PortAudioDependency(AudacityDependency):
    def __init__(self, package_options: dict = None):
        super().__init__("portaudio", "19.7.0", package_options=package_options)

    def apply_options(self, conanfile, package):
        super().apply_options(conanfile, package)

        if conanfile.settings.os == "Windows":
            package.with_asio = conanfile.options.use_asio
            package.with_wdmks = False

        if conanfile.settings.os != "Macos":
            package.with_jack = conanfile.options.use_jack


# Curl needs the propper TLS backend set
@dataclass
class CurlDependency(AudacityDependency):
    def __init__(self, package_options: dict = None):
        super().__init__("libcurl", "7.82.0", package_options=package_options)

    def apply_options(self, conanfile, package):
        super().apply_options(conanfile, package)

        if conanfile.settings.os == "Windows":
            package.with_ssl = "schannel"
        elif conanfile.settings.os == "Macos":
            package.with_ssl = "darwinssl"
        else:
            package.with_ssl = "openssl"



class AudacityConan(ConanFile):
    settings = "os", "compiler", "build_type", "arch"
    generators = "CMakeDeps"
    options = { "use_asio": [True, False], "use_jack": [True, False], "lib_dir": [None, "ANY"]}
    default_options = { "use_asio": False, "use_jack": False, "lib_dir": None }

    # List of Audacity dependencies
    _dependencies = [
        AudacityDependency("zlib", "1.2.13"),
        AudacityDependency("libpng", "1.6.39"),
        AudacityDependency("expat", "2.5.0"),
        AudacityDependency("libjpeg-turbo", "2.1.5"),
        wxWidgetsAudacityDependency(),

        AudacityDependency("libmp3lame", "3.100"),
        AudacityDependency("mpg123", "1.31.2", package_options={ "network": False }),
        AudacityDependency("libid3tag", "0.15.2b", package_options={ "shared": False }),
        AudacityDependency("wavpack", "5.6.0"),
        AudacityDependency("ogg", "1.3.5"),
        AudacityDependency("flac", "1.4.2"),
        AudacityDependency("opus", "1.4.0", override=True),
        AudacityDependency("opusfile", "0.12", package_options={ "shared": False, "http": False }),
        AudacityDependency("vorbis", "1.3.7"),
        AudacityDependency("libsndfile", "1.0.31", package_options={ "programs": False }),

        AudacityDependency("vst3sdk", "3.7.7"),

        AudacityDependency("libuuid", "1.0.3"),

        PortAudioDependency(),

        AudacityDependency("portmidi", "r234"),

        AudacityDependency("threadpool", "20140926"),
        CurlDependency(),

        AudacityDependency("rapidjson", "1.1.0"),

        AudacityDependency("breakpad", "2023.01.27"),

        CrashpadDependency("cci.20220219-audacity"),

        AudacityDependency("catch2", "2.13.8")
    ]

    options.update({f"use_{dependency.name}": [True, False] for dependency in _dependencies})
    default_options.update({f"use_{dependency.name}": dependency.default_enabled for dependency in _dependencies})

    def requirements(self):
        for dependency in self._dependencies:
            if getattr(self.options, f"use_{dependency.name}"):
                if dependency.override:
                    self.requires(dependency.reference(self), override=True)
                else:
                    self.requires(dependency.reference(self))

    def build_requirements(self):
        if self.settings.os not in ["Windows", "Macos"]:
            self.build_requires("patchelf/0.13@audacity/stable")

    def configure(self):
        self.options["*"].shared = True

        for dependency in self._dependencies:
            if getattr(self.options, f"use_{dependency.name}"):
                self.output.info(f"Applying options for {dependency.name}...")
                dependency.apply_options(self, self.options[dependency.name])

    def layout(self):
        cmake_layout(self, build_folder="")
        self.folders.generators = "generators"

    def generate(self):
        deps_lookup = { dependency.name: dependency for dependency in self._dependencies }

        for dep in self.dependencies.host.values():
            self.output.info(f"Copying files for {dep.ref.name}...")
            if dep.ref.name in deps_lookup:
                deps_lookup[dep.ref.name].copy_files(self, dep)
            else:
                global_copy_files(self, dep)



import configparser
import sys
import os

from helpers.version import version_tuple
from helpers.directories import directories

__arch_map = {
    'x86_64': 'x86_64',
    'amd64': 'x86_64',
    'x64': 'x86_64',
    'x86': 'x86',
    'arm64': 'armv8',
    'aarch64': 'armv8',
    'arm': 'armv7',
}

__compilers_map = {
    'MSVC': 'msvc',
    'GNU': 'gcc',
    'Clang': 'clang',
    'AppleClang': 'apple-clang',
}

def get_profile_os() -> str:
    try:
        return {
            'win32': 'Windows',
            'cygwin':'Windows',
            'darwin': 'Macos',
            'linux': 'Linux',
            'freebsd': 'FreeBSD',
            'openbsd': 'OpenBSD'
        }[sys.platform]
    except KeyError:
        return sys.platform


def get_conan_compiler(cmake_compiler: str) -> str:
    return __compilers_map.get(cmake_compiler, cmake_compiler)


def get_conan_compiler_version(cmake_compiler: str, cmake_compiler_version: str) -> str:
    compiler_version = version_tuple(cmake_compiler_version)

    if cmake_compiler == 'MSVC':
        if compiler_version < (19, 20, 0, 0):
            raise RuntimeError(f'Visual Studio {compiler_version} is not supported')
        else:
            return str(compiler_version[0]) + str(compiler_version[1] // 10) # 19.35.32217.1 -> 193
    else:
        if cmake_compiler == 'AppleClang' and compiler_version[0] < 13:
            return f'{compiler_version[0]}.{compiler_version[1]}'

        return compiler_version[0]


def get_conan_arch(cmake_arch: str):
    arch = cmake_arch.lower()
    return __arch_map.get(arch, arch)


class Profiles:
    compiler = None

    build_profile_path = None

    base_host_profile_path = None
    options_profile_path = None

    host_profile_paths = []

    def __init__(self, args):
        if not os.path.isdir(directories.profiles_dir):
            os.makedirs(directories.profiles_dir)

        self.compiler = get_conan_compiler(args.compiler)

        if not args.build_profile:
            self.build_profile_path = os.path.join(directories.profiles_dir, 'build.profile')
            if not os.path.isfile(self.build_profile_path):
                print('Generating build profile...')
                self.__detect_base_profile(args, self.build_profile_path, False)
        else:
            self.build_profile_path = args.build_profile

        if not args.host_profile:
            self.base_host_profile_path = os.path.join(directories.profiles_dir, 'host.profile')
            if not os.path.isfile(self.base_host_profile_path):
                print('Generating base host profile...')
                self.__detect_base_profile(args, self.base_host_profile_path, True)
        else:
            self.base_host_profile_path = args.host_profile

        self.options_profile_path = os.path.join(directories.profiles_dir, f'options.profile')
        print('Generating options profile...')
        self.__create_options_profile(args, self.options_profile_path)

        for build_type in ('Debug', 'Release', 'RelWithDebInfo', 'MinSizeRel'):
            profile_path = os.path.join(directories.profiles_dir, f'host-{build_type.lower()}.profile')
            if not os.path.isfile(profile_path):
                print(f'Generating host-{build_type.lower()} profile...')
                self.__create_host_profile(build_type, profile_path)
            self.host_profile_paths.append((build_type, profile_path))

    def __create_host_profile(self, build_type:str, profile_path:str):
        profile = self.__create_profile_stub()

        profile['settings'] = {}
        profile['settings']['&:build_type'] = build_type
        profile['settings']['build_type'] = "Debug" if build_type == "Debug" else "RelWithDebInfo"
        if self.compiler == 'msvc':
            profile['settings']['compiler.runtime_type'] = "Debug" if build_type == "Debug" else "Release"

        overrides_profile = os.path.join(directories.root_dir, f'profile_overrides_{build_type.lower()}.txt')

        with open(profile_path, 'w') as f:
            f.write('# This file is auto-generated. \n\n')

            f.write(f'include({self.base_host_profile_path})\n')
            f.write(f'include({self.options_profile_path})\n')

            if os.path.isfile(overrides_profile):
                f.write(f'include({overrides_profile})\n')

            f.write('\n')
            profile.write(f)

    def __create_profile_stub(self):
        profile = configparser.ConfigParser(allow_no_value=True, delimiters='=')
        profile.optionxform = str
        return profile

    def __create_options_profile(self, args, profile_path:str):
        profile = self.__create_profile_stub()

        profile['options'] = {}

        if args.options:
            for option in args.options:
                option_name, option_value = option.split('=', 1)
                profile['options'][f'&:{option_name}'] = option_value

        if args.lib_dir:
            profile['options']['&:lib_dir'] = args.lib_dir

        with open(profile_path, 'w') as f:
            f.write('# This file is auto-generated. \n\n')
            profile.write(f)

    def __detect_base_profile(self, args, profile_path:str, is_host_profile:bool):
        profile = self.__create_profile_stub()

        profile['settings'] = {
            'os': get_profile_os(),
            'arch': get_conan_arch(args.target_arch if is_host_profile else args.build_arch),
            'compiler': get_conan_compiler(args.compiler),
            'compiler.version': get_conan_compiler_version(args.compiler, args.compiler_version),
            'compiler.cppstd': '17',
        }

        if not is_host_profile:
            profile['settings']['build_type'] = 'Release'

        profile['options'] = {}
        profile['conf'] = {}

        if args.compiler == 'MSVC':
            profile['settings']['compiler.runtime'] = "dynamic"
            if not is_host_profile:
                profile['settings']['compiler.runtime_type'] = "Release"
        elif args.compiler == 'GNU':
            profile['settings']['compiler.libcxx'] = 'libstdc++11'
        elif args.compiler == 'Clang' or args.compiler == 'AppleClang':
            profile['settings']['compiler.libcxx'] = 'libc++'

        if args.min_os_version and is_host_profile:
            # Only set the minimum OS version for host profiles
            profile['settings']['os.version'] = args.min_os_version

        if sys.platform == 'darwin':
            import subprocess
            profile['conf']['tools.apple:sdk_path'] = subprocess.check_output(['xcrun', '--sdk', 'macosx', '--show-sdk-path']).decode('utf-8').strip()
            if not is_host_profile:
                # For build tools - use a higher version of macOS
                profile['settings']['os.version'] = "10.15" if get_conan_arch(args.build_arch) == 'x86_64' else "11.0"

        if args.download_cache:
            profile['conf']['tools.files.download:download_cache']= args.download_cache

        with open(profile_path, 'w') as f:
            f.write('# This file is auto-generated. \n\n')
            profile.write(f)



from stft_log import *

# Now we have a variable called `stft` which is a vector of vectors of floats, each vector of floats is a frame of the STFT.
# We can visualize this using matplotlib's `imshow` function.

# Plot the STFT
import matplotlib.pyplot as plt

# Use imshow, scaling the axes to show Hz on the y axis and seconds on the x axis.
# Transform `stft` to a numpy matrix.
import numpy as np
stft = np.transpose(np.array(stft))

plt.imshow(stft, aspect='auto', cmap='viridis', interpolation='nearest', origin='lower')
plt.colorbar()
plt.show()



import os

class Directories:
    root_dir = os.path.realpath(os.path.join(os.path.dirname(__file__), '..'))

    build_dir = None
    lib_dir = None

    env_dir = None
    conan_home_dir = None

    profiles_dir = None

    def update(self, parsed_args):
        self.build_dir = os.path.realpath(parsed_args.build_dir)

        if parsed_args.lib_dir:
            self.lib_dir = os.path.realpath(parsed_args.lib_dir)

        self.env_dir = os.path.join(self.build_dir, 'conan', 'env')
        self.profiles_dir = os.path.join(self.build_dir, 'conan', 'profiles')


directories = Directories()



#!/usr/bin/env python3
# SPDX-License-Identifier: GPL-3.0-only
# Audacity-CLA-applies
#
# Audacity
# A Digital Audio Editor
#
# Copyright (C) 2024 Audacity Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import io
import sys
import json
import requests

YOUTUBE_API_URL = "https://youtube.googleapis.com/youtube/v3"
MAX_NUMBER_OF_RESULT_ITEMS = 100

class PlaylistItem:
    def __init__(self):
        self.title = ""
        self.author = ""
        self.videoUrl = ""
        self.thumbnailUrl = ""
        self.durationSecs = 0

def videoDurationSecs(durationInIsoFormat):
    # Available ISO8601 duration format: P#Y#M#DT#H#M#S
    import re
    regexp = re.compile(r'('
                        r'P'
                        r'((?P<years>[0-9]+)Y)?'
                        r'((?P<months>[0-9]+)M)?'
                        r'((?P<days>[0-9]+)D)?'
                        r'T'
                        r'((?P<hours>[0-9]+)H)?'
                        r'((?P<minutes>[0-9]+)M)?'
                        r'((?P<seconds>[0-9]+)S)?'
                        r')')

    match = regexp.match(durationInIsoFormat)

    if not match:
        return 0

    hours = int(match.group("hours") or 0)
    minutes = int(match.group("minutes") or 0)
    seconds = int(match.group("seconds") or 0)

    return seconds + minutes * 60 + hours * 60 * 60

def parsePlaylistItemsIds(playlistDoc):
    result = []
    items = playlistDoc["items"]

    for item in items:
        snippetObj = item["snippet"]
        resourceIdObj = snippetObj["resourceId"]
        videoId = resourceIdObj["videoId"]

        result.append(videoId)

    return result

def parseVideosInfo(videosInfoDoc):
    result = []
    items = videosInfoDoc["items"]

    for item in items:
        snippetObj = item["snippet"]

        playlistItem = PlaylistItem()

        playlistItem.title = snippetObj["title"]
        playlistItem.author = snippetObj["channelTitle"]

        playlistItem.url = "https://www.youtube.com/watch?v=" + item["id"]

        thumbnailsObj = snippetObj["thumbnails"]
        thumbnailsMediumObj = thumbnailsObj["medium"]
        playlistItem.thumbnailUrl = thumbnailsMediumObj["url"]

        contentDetails = item["contentDetails"]
        durationInIsoFormat = contentDetails["duration"]
        playlistItem.durationSecs = videoDurationSecs(durationInIsoFormat)

        result.append(playlistItem)

    return result

YOUTUBE_API_KEY = sys.argv[1]
PLAYLIST_ID = sys.argv[2]
PLAYLIST_FILE = sys.argv[3]

print("=== Read json file ===")

json_file = open(PLAYLIST_FILE, "r+")
jsonDict = json.load(json_file)
json_file.close()

print("=== Get playlist items ===")

headers = {
    'Accept': 'application/json',
}

url = YOUTUBE_API_URL + f"/playlistItems?part=snippet&playlistId={PLAYLIST_ID}&key={YOUTUBE_API_KEY}&maxResults={MAX_NUMBER_OF_RESULT_ITEMS}"
r = requests.get(url, headers=headers)

playlist_items = json.loads(r.text)

print("=== Parse playlist items ===")

playlist_items_ids = parsePlaylistItemsIds(playlist_items)

print("=== Get videos info ===")

params = f"part=snippet,contentDetails&key={YOUTUBE_API_KEY}&maxResults={MAX_NUMBER_OF_RESULT_ITEMS}"
for item_id in playlist_items_ids:
    params += f"&id={item_id}"

url = YOUTUBE_API_URL + f"/videos?{params}"
r = requests.get(url, headers=headers)

playlist_videos_info = json.loads(r.text)

print("=== Parse videos info ===")

playlist = parseVideosInfo(playlist_videos_info)

for item in playlist:
    new_item = {"title": item.title,
                "author": item.author,
                "url": item.url,
                "thumbnailUrl": item.thumbnailUrl,
                "durationSecs": item.durationSecs }
    jsonDict["default"].append(new_item)

json_file = open(PLAYLIST_FILE, "w")
json_file.write(json.dumps(jsonDict, indent=4))
json_file.close()



import os
import shutil
import time

def safe_rm_tree(path):
    if not os.path.isdir(path):
        return

    def onerror(func, path, exc_info):
        import stat

        # Is the error an access error?
        if not os.access(path, os.W_OK):
            print(f"Failed to remove `{path}`. Trying to change permissions...")
            os.chmod(path, stat.S_IWUSR)
            func(path)
        else:
            raise OSError("Cannot change permissions for {}! Exception info: {}".format(path, exc_info))

    for i in range(20):
        try:
            shutil.rmtree(path, onerror=onerror)
            return
        except Exception as e:
            delay = 0.5 * float(i + 1)
            print(f"Failed to remove `{path}`: `{e}`. Retrying in {delay} seconds...")
            time.sleep(delay)



import sys
import os
sys.dont_write_bytecode = True
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..', "scripts"))

import argparse
import glob
import hashlib
import subprocess

from utils.files import safe_rm_tree

from helpers.conan_environment import get_conan, validate_conan,  ConanEnv
from helpers.directories import directories
from helpers.remotes import validate_remotes
from helpers.profiles import Profiles

required_conan_version = (2, 0, 0)
runner_version = 2


def get_conanfile_path():
    return os.path.join(directories.root_dir, 'conanfile.py')

def get_conanfile_hash():
    with open(get_conanfile_path(), 'rb') as f:
        return hashlib.sha256(f.read()).hexdigest()

def init_args():
    parser = argparse.ArgumentParser(description='Audacity Conan runner')

    parser.add_argument('-b', '--build-dir', help='Build directory (${CMAKE_BINARY_DIR})', required=True)
    parser.add_argument('-o', '--options', help='Conan options', nargs='*')
    parser.add_argument('--force-build', help='Force build', action='store_true')
    parser.add_argument('--compiler', help='Compiler', required=True)
    parser.add_argument('--compiler-version', help='Compiler version', required=True)
    parser.add_argument('--target-arch', help='Target architecture', default="x86_64")
    parser.add_argument('--build-arch', help='Build architecture', default="x86_64")
    parser.add_argument('--download-cache', help='Conan download cache', default=None)
    parser.add_argument('--build-types', help='Build types', default=['Release'], nargs="*")
    parser.add_argument('--lib-dir', help='Directory to copy the shared libraries to', default=None)
    parser.add_argument('--min-os-version', help='Minimum OS version', default=None)
    parser.add_argument('--disallow-prebuilt', help='Disallow prebuilt dependencies', action='store_true')
    parser.add_argument('--build-profile', help='Build profile', default=None)
    parser.add_argument('--host-profile', help='Host profile', default=None)

    return parser.parse_args()


def get_conan_venv(build_dir):
    return os.path.abspath(os.path.join(build_dir, 'conan_venv'))


def generate_args_string(args):
    if args.options is None:
        return ''

    return ';'.join(args.options) + ';' + get_conanfile_hash() + ';' + str(runner_version)


def migrate(args):
    old_profiles = ('profile-build.profile',
                    'profile-host-release.profile', 'profile-host-debug.profile',
                    'profile-host-minsizerel.profile', 'profile-host-relwithdebinfo.profile')

    for profile in old_profiles:
        path = os.path.join(args.build_dir, profile)
        if os.path.isfile(path):
            os.remove(path)

    for dir in glob.glob(os.path.join(directories.build_dir, 'conan-install-*')):
        safe_rm_tree(dir)


if __name__ == '__main__':
    args = init_args()
    directories.update(args)

    migrate(args)

    args_file_path = os.path.join(args.build_dir, 'conan_args.txt')
    args_string = generate_args_string(args)

    if os.path.isfile(args_file_path):
        with open(args_file_path, 'r') as args_file:
            old_args = args_file.read()
            if old_args == args_string:
                print('Skipping conan install, arguments and requirements are the same')
                sys.exit(0)

    conan_version = validate_conan(required_conan_version)
    with ConanEnv():
        print(f'Using Conan version {".".join(map(str, conan_version))} ({get_conan()})', flush=True)
        validate_remotes()
        safe_rm_tree(os.path.join(directories.build_dir, 'generators'))

        profiles = Profiles(args)

        for build_type, host_profile in profiles.host_profile_paths:
            conan_options = [
                get_conan(), 'install', directories.root_dir,
                '--build='*'' if args.force_build else '--build=missing',
                '--output-folder', args.build_dir,
                '--remote', 'audacity-recipes-conan2' if (args.force_build or args.disallow_prebuilt) else 'audacity-binaries-conan2',
                '--profile:build', profiles.build_profile_path,
                '--profile:host', host_profile,
            ]

            subprocess.check_call(conan_options, stdin=subprocess.DEVNULL)

    with open(args_file_path, 'w') as args_file:
        args_file.write(args_string)



import argparse
import os
import subprocess
import re

def is_system_lib(path):
    return path.startswith('/System/Library/') or path.startswith('/usr/lib/')


def macos_collect_rpaths(file):
    result = []

    with subprocess.Popen(['otool', '-l', file], stdout=subprocess.PIPE) as p:
        lines = [line.decode('utf-8').strip() for line in p.stdout.readlines()]
        for line_index in range(len(lines)):
            if lines[line_index] == 'cmd LC_RPATH':
                rpath_match = re.match(r'path\s+(.*)\s+\(', lines[line_index + 2])
                if rpath_match:
                    rpath = rpath_match.group(1)
                    if rpath not in result:
                        result.append(rpath)

            line_index = line_index + 1

    return result


def get_dylib_id(file):
    with subprocess.Popen(['otool', '-D', file], stdout=subprocess.PIPE) as p:
        lines = [line.decode('utf-8').strip() for line in p.stdout.readlines()]
        if len(lines) == 2:
            return lines[1].split('/')[-1]
    return ''


def macos_collect_dependencies(file):
    result = []
    dylib_id = get_dylib_id(file)

    with subprocess.Popen(['otool', '-L', file], stdout=subprocess.PIPE) as p:
        lines = [line.decode('utf-8').strip() for line in p.stdout.readlines()]
        for line in lines:
            match = re.match(r'(.*)\s+\(', line)

            if not match:
                continue

            lib_line = match.group(1)
            if not lib_line.startswith('@loader_path') and not is_system_lib(lib_line):
                name = lib_line.split('/')[-1]
                if name != dylib_id:
                    result.append((name, lib_line))

    return result


def run_install_name_tool(file, dep_path_prefix):
    dependencies = macos_collect_dependencies(file)
    rpaths = macos_collect_rpaths(file)

    if len(dependencies) == 0 and len(rpaths) == 0:
        return

    install_name_tool = ['install_name_tool']

    for name, lib_line in dependencies:
        install_name_tool.append('-change')
        install_name_tool.append(lib_line)
        install_name_tool.append(dep_path_prefix + '/' + name)

    for rpath in rpaths:
        install_name_tool.append('-delete_rpath')
        install_name_tool.append(rpath)

    install_name_tool.append(file)

    print(' '.join(install_name_tool))
    try:
        subprocess.check_call(install_name_tool)
    except subprocess.CalledProcessError as err:
        print(f'=========\ninstall_name_tool failed with code {err.returncode}\n\tstdout: {err.stdout}\n\tstderr: {err.stderr}\n=========')
        if err.returncode != -9:
            raise

        print("install_name_tool was killed. Retrying with x86_64 architecture...")
        install_name_tool = ["arch", "-arch", "x86_64"] + install_name_tool
        try:
            subprocess.run(install_name_tool, capture_output=True, check=True, text=True)
        except subprocess.CalledProcessError as inner_err:
            print(f'=========\ninstall_name_tool failed with code {inner_err.returncode}\n\tstdout: {inner_err.stdout}\n\tstderr: {inner_err.stderr}\n=========')
            raise


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('bundle', help='The bundle to fix')
    parser.add_argument('-config', help='Configuration type')
    args = parser.parse_args()

    if (args.config == "Debug"):
       exit(0)

    bundle = args.bundle

    if not os.path.exists(bundle):
        print('Bundle not found: ' + bundle)
        exit(1)

    if not os.path.isdir(bundle):
        run_install_name_tool(bundle, '@loader_path/../Frameworks')

        if bundle.find('modules') != -1:
            exit(0)

        bundle = os.path.join(os.path.dirname(args.bundle), '..', '..')
        bundle = os.path.abspath(bundle)

    print(f"Fixing bundle {bundle}")

    frameworks_dir = os.path.join(bundle, 'Contents', 'Frameworks')

    if not os.path.exists(frameworks_dir):
        print('Bundle does not contain a Frameworks directory: ' + bundle)
        exit(1)

    for file in os.listdir(frameworks_dir):
        file_path = os.path.join(frameworks_dir, file)
        if os.path.isfile(file_path):
            run_install_name_tool(file_path, '@loader_path')
